@@ -2,6 +2,10 @@ import os
 import itertools
 import multiprocessing as mp
 import zlib
+import requests
+from bs4 import BeautifulSoup
+import pandas as pd
+import numpy as np
 
 from .equity_curves import *
 from .signals import *
@@ -45,17 +49,24 @@ def path_reference_code(save_name):
     reference_code = str(zlib.crc32(bytes(save_name, 'UTF-8')))[:6]
     return reference_code
 
-def generate_filepath(para_combination,folder=''):
+def generate_filepath(para_combination,folder='',file_format='parquet'):
+
+    if not file_format == 'parquet':
+        file_format = 'csv'
+
     save_name = filename_only(para_combination)
 
     reference_code = path_reference_code(save_name)
     save_name = reference_code + '_' + save_name[:-1]     # position -1 to avoid '&' at the end
 
     output_folder = para_combination['output_folder']
-    filepath = os.path.join(folder,output_folder, f'{save_name}.csv')
+    filepath = os.path.join(folder,output_folder, f'{save_name}.{file_format}')
 
     return filepath
 
+# def read_data(path):
+
+
 
 def mp_cal_performance(tuple_data):
     para_combination = tuple_data[0]
@@ -66,11 +77,34 @@ def mp_cal_performance(tuple_data):
     manager_list.append((index,result))
 
 
-def generate_backtest_result(all_para_combination, number_of_core=8):
+def generate_backtest_result(all_para_combination, number_of_core=8, risk_free_rate='geometric_mean'):
+
+
+    ## Get / Calculate risk free rate
+    start_date = all_para_combination[0]['start_date']
+    end_date = all_para_combination[0]['end_date']
+
+    if isinstance(risk_free_rate, str):
+        try:
+            if risk_free_rate == 'geometric_mean':
+                start_date_year = datetime.datetime.strptime(start_date, '%Y-%m-%d').year
+                end_date_year = datetime.datetime.strptime(end_date, '%Y-%m-%d').year
+                risk_free_rate = plotguy.get_geometric_mean_of_yearly_rate(start_date_year, end_date_year)
+            else:
+                risk_free_rate = plotguy.get_latest_fed_fund_rate()
+        except:
+            risk_free_rate = 2  # if network error, set rate to 2 %
+            print('Network error. Risk free rate: {:.2f} %'.format(risk_free_rate))
+    else:
+        print('Risk free rate: {:.2f} %'.format(risk_free_rate))
+
+
+
     manager_list = mp.Manager().list()# To save the result with index number
 
     cal_performance_list = []
     for para_combination in all_para_combination:
+        para_combination['risk_free_rate'] = risk_free_rate
         cal_performance_list.append((para_combination, manager_list))
 
     pool = mp.Pool(processes=number_of_core)
@@ -112,17 +146,54 @@ def plot_signal_analysis(py_filename, output_folder, start_date, end_date, para_
     return app
 
 
-def plot(mode, all_para_combination={},  subchart_settings={}):
+def plot(mode, all_para_combination={},  subchart_settings={}, number_of_curves=20, risk_free_rate='geometric_mean'):
     if mode == 'equity_curves':
         result_df = pd.read_csv('backtest_result.csv', index_col=0)
-        app = equity_curves.Plot(all_para_combination, result_df, subchart_settings)
+        app = equity_curves.Plot(all_para_combination, result_df, subchart_settings, number_of_curves)
+
     if mode == 'aggregate':
-        app = aggregate.Aggregate()
+
+        app = aggregate.Aggregate(risk_free_rate)
 
 
     return app
 
 
+def get_latest_fed_fund_rate():
+    url = "https://fred.stlouisfed.org/series/FEDFUNDS"
+    page = requests.get(url)
+    soup = BeautifulSoup(page.content, "html.parser")
+
+    fed_funds_rate = soup.find("span", class_="series-meta-observation-value").text
+    print("Latest Federal Funds Rate:", fed_funds_rate, '%')
+    # fed_funds_rate = float(fed_funds_rate) / 100
+    fed_funds_rate = round( float(fed_funds_rate) ,2)
+    return fed_funds_rate
+
+
+def get_geometric_mean_of_yearly_rate(start_year, end_year): # backtest period
+    url = "https://fred.stlouisfed.org/graph/fredgraph.csv?id=DTB3"
+    response = requests.get(url)
+    data = response.text.split("\n")[:-1]
+    data = [row.split(",") for row in data]
+    df = pd.DataFrame(data[1:], columns=data[0])
+    df.columns = ["date", "risk_free_rate"]
+    df["date"] = pd.to_datetime(df["date"])
+    df["risk_free_rate"] = pd.to_numeric(df["risk_free_rate"], errors='coerce')
+    df.dropna(subset=['risk_free_rate'], inplace=True)
+
+    risk_free_rate_history_yearly = df.resample("A", on="date").mean()
+    risk_free_rate_history_yearly = risk_free_rate_history_yearly.round(3)
+
+    # show only start between start_year and end_year
+    risk_free_rate_history_yearly = risk_free_rate_history_yearly[risk_free_rate_history_yearly.index.year >= start_year]
+    risk_free_rate_history_yearly = risk_free_rate_history_yearly[risk_free_rate_history_yearly.index.year <= end_year]
+
+    fed_fund_rate_geometric_mean = np.exp(np.log(risk_free_rate_history_yearly["risk_free_rate"]).mean())
+    fed_fund_rate_geometric_mean = round(fed_fund_rate_geometric_mean,2)
+    print("Federal Funds Rate Geometric mean from {} to {}: {} %".format(start_year, end_year, fed_fund_rate_geometric_mean))
+
+    return fed_fund_rate_geometric_mean
 
 
 def calculate_mdd(df_csv, col):
@@ -170,7 +241,7 @@ def calculate_win_rate(df_csv):
     return win_rate_dict
 
 
-def calculate_sharp_ratio(df_csv, col):
+def calculate_sharp_ratio(df_csv, col, risk_free_rate):
     df = df_csv.copy()
     holding_period_day = (df.loc[df.index[-1], 'date'] - df.loc[df.index[0], 'date']).days
     net_profit = df.at[df.index[-1], col] - df.at[df.index[0], col]
@@ -184,7 +255,7 @@ def calculate_sharp_ratio(df_csv, col):
     annualized_std = equity_value_pct_series.std() * math.sqrt(365)
 
     if annualized_std > 0:
-        annualized_sr = annualized_return / annualized_std
+        annualized_sr = (annualized_return - float(risk_free_rate)/100 ) / annualized_std
     else:
         annualized_sr = 0
 
@@ -205,13 +276,18 @@ def resample_summary_to_daily(para_combination, folder=''):
     lot_size = para_combination['lot_size']
     intraday = para_combination['intraday']
     freq = para_combination['freq']
+    file_format = para_combination['file_format']
     sec_profile = para_combination['sec_profile']
     margin_req = sec_profile['margin_req']
     multiplier = sec_profile['multiplier']
 
     # Read backtest data
-    save_path = generate_filepath(para_combination=para_combination,folder=folder)
-    df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+    save_path = generate_filepath(para_combination=para_combination,folder=folder,file_format=file_format)
+    if file_format == 'parquet':
+        df_backtest = pd.read_parquet(save_path) # Daraframe that may not be daily
+    else:
+        df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+
     df_backtest['date'] = pd.to_datetime(df_backtest['date'], format='%Y-%m-%d')
     df_backtest = df_backtest.loc[(df_backtest['date'] >= start_date) & (df_backtest['date'] <= end_date)]
     df_backtest = df_backtest.reset_index(drop=True)  # Reset Index
@@ -353,14 +429,22 @@ def cal_performance(para_combination):
     start_date = para_combination['start_date']
     end_date = para_combination['end_date']
 
+    risk_free_rate = para_combination['risk_free_rate']
+
     intraday = para_combination['intraday']
     summary_mode = para_combination['summary_mode']
 
+    file_format = para_combination['file_format']
+
     if (intraday or summary_mode):
         df_daily = resample_summary_to_daily(para_combination=para_combination)
     else:
-        save_path = generate_filepath(para_combination=para_combination)
-        df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+        save_path = generate_filepath(para_combination=para_combination, file_format=file_format)
+        if file_format == 'parquet':
+            df_backtest = pd.read_parquet(save_path)  # Daraframe that may not be daily
+        else:
+            df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+
         df_backtest['date'] = pd.to_datetime(df_backtest['date'], format='%Y-%m-%d')
         df_backtest = df_backtest.loc[(df_backtest['date'] >= start_date) & (df_backtest['date'] <= end_date)]
         df_backtest = df_backtest.reset_index(drop=True)  # Reset Index
@@ -379,21 +463,26 @@ def cal_performance(para_combination):
     df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
 
     # Strategy Performance
-    net_profit, holding_period_day, return_on_capital, annualized_return, annualized_std, annualized_sr = calculate_sharp_ratio(df, 'equity_value')
+    net_profit, holding_period_day, return_on_capital, annualized_return, annualized_std, annualized_sr = calculate_sharp_ratio(df, 'equity_value', risk_free_rate)
     mdd_pct, mdd_dollar = calculate_mdd(df, 'equity_value')
     mdd_pct = mdd_pct * -100
     mdd_dollar = mdd_dollar * -1
 
     # Win Rate (need to use df_backtest directly)
-    save_path = generate_filepath(para_combination=para_combination)
-    df_backtest = pd.read_csv(save_path, index_col=0)
+
+    save_path = generate_filepath(para_combination=para_combination, file_format=file_format)
+    if file_format == 'parquet':
+        df_backtest = pd.read_parquet(save_path)  # Daraframe that may not be daily
+    else:
+        df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+
     win_rate_dict = calculate_win_rate(df_backtest)
     num_of_trade, num_of_loss, num_of_win, win_rate, loss_rate = win_rate_dict['Overall']
     # print('net_profit',net_profit)
 
     # BaH Performance
     bah_net_return, holding_period_day, bah_return, \
-    bah_annualized_return, bah_annualized_std, bah_annualized_sr = calculate_sharp_ratio(df, 'close')
+    bah_annualized_return, bah_annualized_std, bah_annualized_sr = calculate_sharp_ratio(df, 'close', risk_free_rate)
     initial_capital = df.loc[df.index[0], 'equity_value']
     df['bah_equity_curve'] = df['close'] * initial_capital // df.loc[df.index[0], 'close']
     bah_mdd_pct, bah_mdd_dollar = calculate_mdd(df, 'bah_equity_curve')
@@ -450,6 +539,7 @@ def cal_performance(para_combination):
             result_dict[f'{year}_win_rate'] = '--'
 
     result_dict['return_to_bah'] = return_on_capital - bah_return
+    result_dict['risk_free_rate'] = risk_free_rate
 
     return result_dict
 
