@@ -19,16 +19,7 @@ import dash_dangerously_set_inner_html   # for all Plot Class
 class Components:
     chart_bg = '#1f2c56'
 
-    sort_method_dropdown = html.Div(
-        dbc.Select(id='sort_method',
-                   placeholder="Select Sorting Method",
-                   value='Top 20 Net Profit',
-                   options=[{'label': 'Top 20 Net Profit', 'value': 'Top 20 Net Profit'},
-                            {'label': 'Top 20 Return-BaH % Difference', 'value': 'Top 20 Return to BaH Ratio'},
-                            {'label': 'Top 20 Net Profit/MDD Ratio', 'value': 'Top 20 Net Profit to MDD'}, ],
-                   style={'border-radius': '5px',   'font-size': '12px', }),
-        style={'padding-left': '15px', 'width': '235px'})
-
+    sort_method_dropdown = html.Div()
 
     filter_dropdown = html.Div(id='filter_dropdown',
                                children=dbc.Select(id='filter_name',
@@ -78,6 +69,17 @@ class Components:
                                  'border-radius': '5px', 'text-align': 'center',
                                  'font-size': '13px', 'height':'30px'}
 
+    def __init__(self,number_of_curves):
+        self.sort_method_dropdown = html.Div(
+            dbc.Select(id='sort_method',
+                       placeholder="Select Sorting Method",
+                       value=f'Top Net Profit',
+                       options=[{'label': f'Top {number_of_curves} Net Profit', 'value': 'Top Net Profit'},
+                                {'label': f'Top {number_of_curves} Return-BaH % Difference', 'value': 'Top Return to BaH Ratio'},
+                                {'label': f'Top {number_of_curves} Net Profit/MDD', 'value': 'Top Net Profit to MDD'}, ],
+                       style={'border-radius': '5px', 'font-size': '12px', }),
+            style={'padding-left': '15px', 'width': '235px'})
+
 
     def empty_line_chart(self):
         chart_bg = self.chart_bg
@@ -197,7 +199,7 @@ class Components:
         return radioitems_div
 
 
-    def update_performance_matrix(self, start_date,end_date,df,para_dict):
+    def update_performance_matrix(self, start_date,end_date,df,para_dict, risk_free_rate, ref_code):
         per_col1 =[]
         per_col2 = []
         per_col_1_1 = []
@@ -262,11 +264,12 @@ class Components:
 
         per_col_1_1.append(html.Div('Number of Trade'))
         per_col_1_1.append(html.Div('Net Profit'))
-        per_col_1_1.append(html.Div('Net Profit/MDD Ratio'))
+        per_col_1_1.append(html.Div('Net Profit/MDD'))
         per_col_1_1.append(html.Div('Return-BaH % Diff.'))
         per_col_1_1.append(html.Div('Win Rate'))
         per_col_1_1.append(html.Div('COV'))
         per_col_1_1.append(html.Div('Total Commission'))
+        per_col_1_1.append(html.Div('Risk Free Rate'))
 
         per_col_2_1.append(html.Div('Return on Capital'))
         per_col_2_1.append(html.Div('Ann. Return'))
@@ -277,6 +280,11 @@ class Components:
 
         for i in range(7):
             per_col_1_2.append(html.Div(df[keys[i]], style={'text-align': 'center'}))
+        try:
+            per_col_1_2.append(html.Div( "{:.2%}".format(risk_free_rate / 100) , style={'text-align': 'center'}))
+        except:
+            per_col_1_2.append(html.Div('-----', style={'text-align': 'center'}))
+
         for i in range(7, 13):
             per_col_2_2.append(html.Div(str(df[keys[i]]), style={'text-align': 'center'}))
         for i in range(13, 19):
@@ -347,6 +355,11 @@ class Components:
             html.Div('Performance by Year', style={'color': 'Cyan', 'font-size': '15px'}),
             html.Div(style={'height': '5px', }),
             html.Div(children=year_col1, style={'font-size': '11px'}),
+
+            html.Div(style={'height': '10px'}),
+
+            html.Div(f'Ref: {ref_code}', style={'font-size': '11px'}),
+
             html.Div(style={'height': '20px'}),
 
             html.Div([
@@ -357,6 +370,7 @@ class Components:
 
         ],style={'padding': '0px',})
 
+
         return matrix_div
 
 
@@ -395,13 +409,13 @@ class Components:
         return filter_button
 
 
-    def sort_method_df(self, sort_method, result_df):
-        if sort_method == 'Top 20 Net Profit':
-            df_sorted = result_df.sort_values(by='net_profit', ascending=False).head(20).copy()
-        elif sort_method == 'Top 20 Net Profit to MDD':
-            df_sorted = result_df.sort_values(by='net_profit_to_mdd', ascending=False).head(20).copy()
-        elif sort_method == 'Top 20 Return to BaH Ratio':
-            df_sorted = result_df.sort_values(by='return_to_bah', ascending=False).head(20).copy()
+    def sort_method_df(self, sort_method, result_df, number_of_curves):
+        if sort_method == 'Top Net Profit':
+            df_sorted = result_df.sort_values(by='net_profit', ascending=False).head(number_of_curves).copy()
+        elif sort_method == 'Top Net Profit to MDD':
+            df_sorted = result_df.sort_values(by='net_profit_to_mdd', ascending=False).head(number_of_curves).copy()
+        elif sort_method == 'Top Return to BaH Ratio':
+            df_sorted = result_df.sort_values(by='return_to_bah', ascending=False).head(number_of_curves).copy()
         else:
             df_sorted = result_df.copy()
 
@@ -427,12 +441,17 @@ class Components:
         elif profile == 5:  rgb = (int(252 * degree), 0, 252)
         return 'rgb' + str(rgb)
 
-    def generate_chart_3(self, para_combination, chart3_start, chart3_end, freq):
+    def generate_chart_3(self, para_combination, chart3_start, chart3_end, freq, file_format):
 
         data_folder = para_combination['data_folder']
         code = para_combination['code']
         data_path = os.path.join(data_folder, code + '.csv')
-        df = pd.read_csv(data_path)
+        # df = pd.read_csv(data_path)
+        if file_format == 'parquet':
+            df = pd.read_parquet(data_path)  # Daraframe that may not be daily
+        else:
+            df = pd.read_csv(data_path, index_col=0)
+
 
         df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
         df['datetime'] = pd.to_datetime(df['datetime'], format='%Y-%m-%d %H:%M:%S')
@@ -445,8 +464,12 @@ class Components:
         # isolate missing timestamps
         dt_breaks = [d for d in dt_all.strftime("%Y-%m-%d %H:%M:%S").tolist() if not d in dt_obs]
 
-        save_path = plotguy.generate_filepath(para_combination)
-        df_signal_list = pd.read_csv(save_path)
+        save_path = plotguy.generate_filepath(para_combination=para_combination, file_format=file_format)
+        if file_format == 'parquet':
+            df_signal_list = pd.read_parquet(save_path)  # Daraframe that may not be daily
+        else:
+            df_signal_list = pd.read_csv(save_path, index_col=0)
+
         df_signal_list['date'] = pd.to_datetime(df_signal_list['date'], format='%Y-%m-%d')
         df_signal_list['datetime'] = pd.to_datetime(df_signal_list['datetime'], format='%Y-%m-%d %H:%M:%S')
         df_signal_list = df_signal_list.loc[
@@ -552,7 +575,7 @@ class Components:
 
             line_colour = graph_df.loc[i].line_colour
 
-
+            file_format = para_combination['file_format']
 
             # df = pd.read_csv(save_path)
             # df['date'] = pd.to_datetime(df['date'], format='%Y-%m-%d')
@@ -560,11 +583,12 @@ class Components:
             if (intraday or summary_mode):
                 df_daily = plotguy.resample_summary_to_daily(para_combination=para_combination)
             else:
-                save_path = plotguy.generate_filepath(para_combination)
-                df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
-                # df_backtest['date'] = pd.to_datetime(df_backtest['date'], format='%Y-%m-%d')
-                # df_backtest = df_backtest.loc[(df_backtest['date'] >= start_date) & (df_backtest['date'] <= end_date)]
-                # df_backtest = df_backtest.reset_index(drop=True)  # Reset Index
+                save_path = plotguy.generate_filepath(para_combination=para_combination, file_format=file_format)
+                if file_format == 'parquet':
+                    df_backtest = pd.read_parquet(save_path)  # Daraframe that may not be daily
+                else:
+                    df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+
                 df_daily = df_backtest.copy()
 
             start_date = para_combination['start_date']
@@ -686,8 +710,15 @@ class Components:
 
 
     def generate_chart_2_full(self, para_combination, line_colour, settings):
-        save_path = plotguy.generate_filepath(para_combination)
-        df_csv = pd.read_csv(save_path)
+        file_format = para_combination['file_format']
+
+        save_path = plotguy.generate_filepath(para_combination=para_combination, file_format=file_format)
+        if file_format == 'parquet':
+            df_backtest = pd.read_parquet(save_path)  # Daraframe that may not be daily
+        else:
+            df_backtest = pd.read_csv(save_path, index_col=0)  # Daraframe that may not be daily
+
+        df_csv = df_backtest.copy()
         df_chart = self.prepare_df_chart(df_csv)
 
         df_signal = df_chart.copy()
@@ -801,14 +832,28 @@ class Components:
         fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
                                       x=df_chart['date'], y=df_chart['equity_value'],
                                       line=dict(color=line_colour, width=1), name='Strategy Equity'), row=1, col=1)
-        fig_line.add_trace(go.Scatter(mode='lines', hoverinfo='skip',
+
+        hover_template_stock_price = "<br>".join([
+            "Close : %{customdata[0]}",
+        ])
+
+        df_close = pd.DataFrame()
+        df_close['close'] = df_backtest['close']
+
+        hover_template_stock_price = "<br>".join([
+            "Stock Price: %{customdata[0]}",
+        ])
+
+        fig_line.add_trace(go.Scatter(mode='lines', customdata=df_close, hovertemplate=hover_template_stock_price,
+                                      # hoverinfo='skip',
                                       x=df_chart['date'], y=df_chart['bah'],
-                                      line=dict(color='Grey', width=1), name='BnH Equity'),row=1, col=1 )
+                                      line=dict(color='Grey', width=1), name=''),row=1, col=1 )
         fig_line.update_layout(plot_bgcolor=self.chart_bg, paper_bgcolor=self.chart_bg, height=500,
                                margin=dict(l=85, r=25, t=35, b=20),
                                font={"color": "white", 'size': 9}, yaxis={'title': 'Equity'},
                                xaxis={'title': ''}
                                )
+
         fig_line.add_trace(go.Scatter(mode='markers', customdata=df_signal_final, hovertemplate=hover_template,
                                       x=df_chart['date'], y=df_chart['open'], # visible='legendonly',
                                       marker=dict(color='rgba(0, 0, 0, 0)', size=9,
@@ -911,8 +956,8 @@ class Components:
         return fig_pct, fig_rise, fig_fall
 
 
-    def path_to_dict(self,path):
-        filename = 'file=' + path.split('file=')[-1].split('.csv')[0]
+    def path_to_dict(self,path, file_format):
+        filename = 'file=' + path.split('file=')[-1].split(f'.{file_format}')[0]
         parameters = {}
         for element in filename.split('&'):
             key, content = element.split('=')
@@ -962,8 +1007,15 @@ class Components:
             df_saved = pd.read_csv(f'{folder}/saved_strategies.csv')
             for i, row in df_saved.iterrows():
                 equity_path = f'{folder}/{row["path"]}'
-                parameters = self.path_to_dict(equity_path)
+
                 dict = {}
+                dict['para_combination'] = ast.literal_eval(row['para_combination'])
+                para_combination = dict['para_combination']
+                file_format = para_combination['file_format']
+                save_name = plotguy.filename_only(para_combination)
+                ref_code = plotguy.path_reference_code(save_name)
+                parameters = self.path_to_dict(equity_path, file_format )
+
                 dict['folder'] = folder
                 dict['py'] = parameters['file']
                 dict['path'] = equity_path
@@ -980,7 +1032,9 @@ class Components:
                     'total_commission': row['total_commission'],
                     'sharp_ratio': row['sharp'],
                 }
-                dict['para_combination'] = ast.literal_eval(row['para_combination'])
+                dict['format'] = file_format
+                dict['ref'] = ref_code
+
                 start_date_year = datetime.datetime.strptime(parameters['startdate'], '%Y%m%d').year
                 end_date_year = datetime.datetime.strptime(parameters['enddate'], '%Y%m%d').year
                 year_list = list(range(start_date_year, end_date_year + 1))
@@ -991,6 +1045,7 @@ class Components:
 
                 equity_curves.append(dict)
 
+
         line_colour = []
         for c in range(len(equity_curves)):
             profile = c % 6
@@ -1084,6 +1139,28 @@ class Components:
                                 dbc.Col([html.Div(performance_text, style={'line-height': line_height})],
                                         width=6),
                             ]),
+
+                            html.Div(style={'height': '5px'}),
+
+                            dbc.Row([
+                                dbc.Col([html.Div('format', style={'line-height': line_height})],
+                                        width=3),
+                                dbc.Col([html.Div(df.iloc[i - 1].format, style={'line-height': line_height})],
+                                        width=6),
+                            ]),
+
+                            html.Div(style={'height': '5px'}),
+
+                            dbc.Row([
+                                dbc.Col([html.Div('ref', style={'line-height': line_height})],
+                                        width=3),
+                                dbc.Col([html.Div(df.iloc[i - 1].ref, style={'line-height': line_height})],
+                                        width=6),
+                            ]),
+
+                            html.Div(style={'height': '5px'}),
+
+
                         ],width=12),
 
                     ]),
@@ -1120,7 +1197,7 @@ class Components:
         return df, row
 
 
-    def aggregate_performance(self, total_dict, year_list):
+    def aggregate_performance(self, total_dict, year_list, risk_free_rate):
         num_of_trade = "{:,}".format(int(total_dict['num_of_trade']))
         net_profit = "{:,}".format(int(round(total_dict['net_profit'],0)) )
         net_profit_to_mdd = round(total_dict['net_profit_to_mdd'],2)
@@ -1129,11 +1206,37 @@ class Components:
         except:
             win_rate = '--'
 
+        # Sharp Ratio
+        annualized_std = total_dict['annualized_std']
+        annualized_return = total_dict['annualized_return']
+
+        if isinstance(risk_free_rate, str):
+            try:
+                if risk_free_rate == 'geometric_mean':
+                    start_date_year = year_list[0]
+                    end_date_year = year_list[-1]
+                    risk_free_rate_float = plotguy.get_geometric_mean_of_yearly_rate(start_date_year, end_date_year)
+                else:
+                    risk_free_rate_float = plotguy.get_latest_fed_fund_rate()
+            except:
+                risk_free_rate_float = 2  # if network error, set rate to 2 %
+                print('Network error. Risk free rate: {:.2f} %'.format(risk_free_rate_float))
+        else:
+            risk_free_rate_float = risk_free_rate
+            print('Risk free rate: {:.2f} %'.format(risk_free_rate_float))
+
+        if annualized_std > 0:
+            annualized_sr = (annualized_return - float(risk_free_rate_float) / 100) / annualized_std
+        else:
+            annualized_sr = 0
+
+
+
         total_commission = "{:,}".format(int(total_dict['total_commission']))
         return_on_capital = "{:.0%}".format(total_dict['return_on_capital'])
         annualized_return = "{:.0%}".format(total_dict['annualized_return'])
         annualized_std = "{:.0%}".format(total_dict['annualized_std'])
-        annualized_sr = round(total_dict['annualized_sr'],2)
+        annualized_sr = round(annualized_sr,2)
         mdd_dollar = "{:,}".format(int(total_dict['mdd_dollar']))
         mdd_pct = "{:.0%}".format(total_dict['mdd_pct'])
 
@@ -1150,8 +1253,6 @@ class Components:
 
         cov = round(np.std(year_count) / np.mean(year_count), 2)
 
-
-
         div = html.Div([
 
             html.Div('Performance', style={'color': 'Cyan', 'font-size': '15px'}),
@@ -1161,16 +1262,19 @@ class Components:
             dbc.Row([
                 dbc.Col([html.Div('Number of Trade'),
                          html.Div('Net Profit'),
-                         html.Div('Net Profit/MDD Ratio'),
+                         html.Div('Net Profit/MDD'),
                          html.Div('Win Rate'),
                          html.Div('COV'),
                          html.Div('Total Commission'),
+                         html.Div('Risk Free Rate'),
+                         html.Div(style={'height': '10px'}),
                          html.Div('Return on Capital'),
                          html.Div('Ann. Return'),
                          html.Div('Ann. Std'),
                          html.Div('Ann. Sharp Ratio'),
                          html.Div('MDD Dollar'),
-                         html.Div('MDD Percentage')],
+                         html.Div('MDD Percentage')
+                         ],
                         style={'padding-left':'13px'},
                         width=7),
                 dbc.Col([html.Div(num_of_trade),
@@ -1179,6 +1283,8 @@ class Components:
                          html.Div(win_rate),
                          html.Div(cov),
                          html.Div(total_commission),
+                         html.Div("{:.2%}".format(risk_free_rate_float / 100)),
+                         html.Div(style={'height': '10px'}),
                          html.Div(return_on_capital),
                          html.Div(annualized_return),
                          html.Div(annualized_std),
@@ -1207,6 +1313,8 @@ class Components:
                 dbc.Col(year_win_rate_col, width=4),
             ], style={'text-align': 'center'})
 
+
+
         ])
 
         return div
\ No newline at end of file
