@@ -47,16 +47,15 @@ class Plot:
     year_win_count = []
 
 
-    def __new__(self, all_para_combination, result_df, settings):
+    def __new__(self, all_para_combination, result_df, settings, number_of_curves):
         # start_date, end_date and para_dict is the same for all combination, ie use the 1st one
         start_date = all_para_combination[0]['start_date']
         end_date = all_para_combination[0]['end_date']
         para_dict = all_para_combination[0]['para_dict']
-        py_filename = all_para_combination[0]['py_filename']
-        output_folder = all_para_combination[0]['output_folder']
         intraday = all_para_combination[0]['intraday']
         summary_mode = all_para_combination[0]['summary_mode']
         freq = all_para_combination[0]['freq']
+        # risk_free_rate = all_para_combination[0]['risk_free_rate']
         self.initial_capital = all_para_combination[0]['sec_profile']['initial_capital']
 
         # For chart3 initial date range
@@ -67,7 +66,7 @@ class Plot:
 
         chart_bg = self.chart_bg
 
-        components = Components()
+        components = Components(number_of_curves)
 
         start_date_year = datetime.datetime.strptime(start_date, '%Y-%m-%d').year
         end_date_year = datetime.datetime.strptime(end_date, '%Y-%m-%d').year
@@ -77,7 +76,7 @@ class Plot:
         checkbox_div = components.update_checkbox_div(para_dict, result_df)
         col_df = result_df.columns.values.tolist()
         df_col = pd.DataFrame([['-----' for column in col_df]], columns=col_df)
-        performance_matrix = components.update_performance_matrix(start_date, end_date, df_col.iloc[0].copy(), para_dict)
+        performance_matrix = components.update_performance_matrix(start_date, end_date, df_col.iloc[0].copy(), para_dict, '-----', '-----')
         filter_div = components.update_filter_div([])
 
 
@@ -305,8 +304,8 @@ class Plot:
                         else:
                             para_combination = self.para_combination
                             hist_style = {'display': 'none'}
-
-                            chart3_chart = components.generate_chart_3(para_combination, chart3_start, chart3_end, freq)
+                            file_format = para_combination['file_format']
+                            chart3_chart = components.generate_chart_3(para_combination, chart3_start, chart3_end, freq, file_format)
 
                             main_chart_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg, 'display': 'none'}
                             chart3_area_style = {'padding': '5px', 'border-radius': '5px', 'background-color': chart_bg,}
@@ -541,7 +540,7 @@ class Plot:
                     df_checked = current_df.copy()
 
                 # Sort
-                graph_df = components.sort_method_df(sort_method, df_checked)
+                graph_df = components.sort_method_df(sort_method, df_checked, number_of_curves)
                 self.graph_df = graph_df
 
                 fig_line = components.generate_chart_1(graph_df, all_para_combination, intraday, summary_mode)
@@ -572,8 +571,7 @@ class Plot:
             if clickData:
                 if self.chart_type == 1:
                     i = clickData['points'][0]['curveNumber'] - 1
-                    performance_matrix = components.update_performance_matrix(start_date, end_date,
-                                                                              self.graph_df.iloc[i].copy(), para_dict)
+
                     self.line_selected = i
                     button_style = {'text-align': 'right','cursor': 'pointer','font-size':'14px'}
 
@@ -582,13 +580,21 @@ class Plot:
                     para_combination = \
                     df_all_para_combination.loc[df_all_para_combination['reference_index'] == reference_index].to_dict(
                         'records')[0]
-                    self.para_combination = para_combination
-                    save_path = plotguy.generate_filepath(para_combination=para_combination)
+
+                    file_format = para_combination['file_format']
+                    save_path = plotguy.generate_filepath(para_combination=para_combination, file_format=file_format)
+                    ref_code = self.graph_df.iloc[i].reference_code
+                    risk_free_rate = self.graph_df.iloc[i].risk_free_rate
+
+                    performance_matrix = components.update_performance_matrix(start_date, end_date,
+                                                                              self.graph_df.iloc[i].copy(), para_dict,
+                                                                              risk_free_rate, ref_code)
 
                     self.save_clicks = 0 # Needed for save button click tracking
 
                     df = self.graph_df.iloc[i].copy()
                     self.save_path = save_path
+                    self.para_combination = para_combination
                     self.net_profit_to_mdd = df['net_profit_to_mdd']
                     self.net_profit = df['net_profit']
                     self.mdd_dollar = df['mdd_dollar']
